package com.garrettwu.joji;

import com.garrettwu.joji.controller.JojiController;
import com.garrettwu.joji.model.JojiModel;
import com.garrettwu.joji.view.JojiView;

/**
 * The main interface for a game.
 *
 * <p>To use the Joji game framework, implement this interface to provide the handlers that
 * take care of user input, rendering, sound, models, etc.</p>
 *
 * @param <M> The type of model.
 */
public interface JojiGame<
    M extends JojiModel,
    V extends JojiView<? super M, ? super C>,
    C extends JojiController<? super M>> {
  /**
   * Creates a new instance of the game model.
   *
   * @return A new instance of the game model.
   */
  M createModel();

  /**
   * Creates a new instance of the game view.
   *
   * @return A new instance of the game view.
   */
  V createView();

  /**
   * Creates a new instance of the game controller.
   *
   * @return A new instance of the game controller.
   */
  C createController();
}
