package com.garrettwu.joji.controller;

import com.garrettwu.joji.model.JojiModel;

/**
 * A Joji controller is the Controller of the MVC pattern.
 *
 * <p>It provides methods for taking actions within the game, such as starting, pausing, resuming,
 * moving the main character to the right, etc.</p>
 *
 * @param <M> The model this controller manages.
 */
public interface JojiController<M extends JojiModel> {
  /**
   * Sets the model.
   *
   * <p>This will be called with a valid model before any other methods.</p>
   *
   * @param model The model.
   */
  void setModel(M model);

  /**
   * Updates the game.
   *
   * @param age The number of milliseconds since the game started.
   */
  void update(long age);
}
