package com.garrettwu.joji.exec;

import com.garrettwu.joji.JojiGame;
import com.garrettwu.joji.controller.JojiController;
import com.garrettwu.joji.model.JojiModel;
import com.garrettwu.joji.util.JojiClock;
import com.garrettwu.joji.view.JojiView;

/**
 * This is the main interface for executing a Joji game.
 *
 * <p>The executor is in charge of running the game's "main-loop."  The main-loop of a game
 * is where all the magic happens.  It is essentially an infinite loop that reads user input,
 * updates game state, and renders.</p>
 */
public interface JojiExecutor<
    M extends JojiModel,
    V extends JojiView<? super M, ? super C>,
      C extends JojiController<? super M>> {
  /**
   * Runs the game.
   *
   * <p>Depending on the implementation, this may start another thread to run the game and return
   * immediately, or it may block until the game has completed.</p>
   *
   * @param game The game to execute.
   * @param clock A clock.
   */
  void execute(JojiGame<M, V, C> game, JojiClock clock);
}
