package com.garrettwu.joji.exec;

import com.garrettwu.joji.JojiGame;
import com.garrettwu.joji.controller.JojiController;
import com.garrettwu.joji.model.JojiModel;
import com.garrettwu.joji.util.JojiClock;
import com.garrettwu.joji.view.JojiView;

/**
 * A game's main loop.
 */
public class JojiMainLoop<
    M extends JojiModel,
    V extends JojiView<? super M, ? super C>,
    C extends JojiController<? super M>> {
  /** The game to run. */
  private final JojiGame<M, V, C> mGame;

  /** A clock. */
  private final JojiClock mClock;

  /** Whether the game is over. */
  private volatile boolean mIsGameOver;

  /**
   * Constructs a main loop to be executed.
   *
   * @param game The game to run.
   * @param clock A clock.
   */
  public JojiMainLoop(final JojiGame<M, V, C> game, final JojiClock clock) {
    mGame = game;
    mClock = clock;
    mIsGameOver = false;
  }

  /** Runs the main loop until the game is over. */
  public void run() {
    final long startTime = mClock.getCurrentTime();
    final M model = mGame.createModel();
    final V view = mGame.createView();
    final C controller = mGame.createController();
    controller.setModel(model);
    while (!mIsGameOver) {
      final long age = mClock.getCurrentTime() - startTime;
      controller.update(age);
      view.processInput(controller);
      view.render(model);
    }
  }

  /** Stops the main loop (causes the <code>run</code> method to return) */
  public void stop() {
    mIsGameOver = true;
  }
}
