// Copyright 2011 Garrett Wu

package com.garrettwu.joji.view;

import com.garrettwu.joji.controller.JojiController;
import com.garrettwu.joji.model.JojiModel;

/**
 * This is the interface for Joji views.
 *
 * <p>The view is the only point of contact with the user.  It receives input events and provides
 * output (usually in the form of visual cues and sound).  It delegates the work of responding to
 * input events to the controller.  It renders the model directly.</p>
 *
 * @param <C>
 */
public interface JojiView<M extends JojiModel, C extends JojiController<?>> {
  /**
   * Process input events.
   *
   * <p>This is called repeatedly by the executor just before rendering the frame.</p>
   *
   * @param controller The game controller.
   */
  void processInput(C controller);

  /**
   * Renders the model.
   *
   * <p>This is called repeatedly by the executor.  In most games, this is used to draw the
   * entire window once per frame.</p>
   *
   * @param model The model to render.
   */
  void render(M model);
}
